@@ -27,16 +27,6 @@ class Test(unittest.TestCase):
 
     """Unit testing the geometry and geo_filter modules"""
 
-    def assert_raises(self, exception, call_able, *args):
-        """assertRaises() has changed from py2.6 to 2.7! Here is an attempt to
-        cover both"""
-        import sys
-        if sys.version_info < (2, 7):
-            self.assertRaises(exception, call_able, *args)
-        else:
-            with self.assertRaises(exception):
-                call_able(*args)
-
     def test_lonlat_precomp(self):
         area_def = geometry.AreaDefinition('areaD', 'Europe (3km, HRV, VTC)', 'areaD',
                                            {'a': '6378144.0',
@@ -730,8 +720,8 @@ class Test(unittest.TestCase):
         self.assertEqual(y__, 0)
 
         lon, lat = p__(999000, -10, inverse=True)
-        self.assert_raises(ValueError, area_def.get_xy_from_lonlat, lon, lat)
-        self.assert_raises(ValueError, area_def.get_xy_from_lonlat, 0., 0.)
+        self.assertRaises(ValueError, area_def.get_xy_from_lonlat, lon, lat)
+        self.assertRaises(ValueError, area_def.get_xy_from_lonlat, 0., 0.)
 
         # Test getting arrays back:
         lons = [lon_ll + eps_lonlat, lon_ur - eps_lonlat]
@@ -746,22 +736,8 @@ class Test(unittest.TestCase):
     def test_get_area_slices(self):
         """Check area slicing."""
         from pyresample import utils
-        area_id = 'cover'
-        area_name = 'Area to cover'
-        proj_id = 'test'
-        x_size = 3712
-        y_size = 3712
-        area_extent = (-5570248.477339261, -5567248.074173444, 5567248.074173444, 5570248.477339261)
-        proj_dict = {'a': 6378169.5, 'b': 6356583.8, 'h': 35785831.0,
-                     'lon_0': 0.0, 'proj': 'geos', 'units': 'm'}
-
-        area_to_cover = utils.get_area_def(area_id,
-                                           area_name,
-                                           proj_id,
-                                           proj_dict,
-                                           x_size, y_size,
-                                           area_extent)
 
+        # The area of our source data
         area_id = 'orig'
         area_name = 'Test area'
         proj_id = 'test'
@@ -777,9 +753,41 @@ class Test(unittest.TestCase):
                                       x_size, y_size,
                                       area_extent)
 
+        # An area that is a subset of the original one
+        area_to_cover = utils.get_area_def(
+            'cover_subset',
+            'Area to cover',
+            'test',
+            proj_dict,
+            1000, 1000,
+            area_extent=(area_extent[0] + 10000,
+                         area_extent[1] + 10000,
+                         area_extent[2] - 10000,
+                         area_extent[3] - 10000))
         slice_x, slice_y = area_def.get_area_slices(area_to_cover)
-        self.assertEqual(slice_x, slice(0, 3712))
-        self.assertEqual(slice_y, slice(0, 3712))
+        self.assertEqual(slice(3, 3709, None), slice_x)
+        self.assertEqual(slice(3, 3709, None), slice_y)
+
+        # An area similar to the source data but not the same
+        area_id = 'cover'
+        area_name = 'Area to cover'
+        proj_id = 'test'
+        x_size = 3712
+        y_size = 3712
+        area_extent = (-5570248.477339261, -5567248.074173444, 5567248.074173444, 5570248.477339261)
+        proj_dict = {'a': 6378169.5, 'b': 6356583.8, 'h': 35785831.0,
+                     'lon_0': 0.0, 'proj': 'geos', 'units': 'm'}
+
+        area_to_cover = utils.get_area_def(area_id,
+                                           area_name,
+                                           proj_id,
+                                           proj_dict,
+                                           x_size, y_size,
+                                           area_extent)
+        slice_x, slice_y = area_def.get_area_slices(area_to_cover)
+        self.assertEqual(slice(46, 3667, None), slice_x)
+        self.assertEqual(slice(52, 3663, None), slice_y)
+
 
         area_to_cover = geometry.AreaDefinition('areaD', 'Europe (3km, HRV, VTC)', 'areaD',
                                                 {'a': 6378144.0,
@@ -798,6 +806,60 @@ class Test(unittest.TestCase):
         self.assertEqual(slice_x, slice(1610, 2343))
         self.assertEqual(slice_y, slice(158, 515, None))
 
+    def test_get_area_slices_nongeos(self):
+        """Check area slicing for non-geos projections."""
+        from pyresample import utils
+
+        # The area of our source data
+        area_id = 'orig'
+        area_name = 'Test area'
+        proj_id = 'test'
+        x_size = 3712
+        y_size = 3712
+        area_extent = (-5570248.477339745, -5561247.267842293, 5567248.074173927, 5570248.477339745)
+        proj_dict = {'a': 6378169.0, 'b': 6356583.8, 'lat_1': 25.,
+                     'lat_2': 25., 'lon_0': 0.0, 'proj': 'lcc', 'units': 'm'}
+        area_def = utils.get_area_def(area_id,
+                                      area_name,
+                                      proj_id,
+                                      proj_dict,
+                                      x_size, y_size,
+                                      area_extent)
+
+        # An area that is a subset of the original one
+        area_to_cover = utils.get_area_def(
+            'cover_subset',
+            'Area to cover',
+            'test',
+            proj_dict,
+            1000, 1000,
+            area_extent=(area_extent[0] + 10000,
+                         area_extent[1] + 10000,
+                         area_extent[2] - 10000,
+                         area_extent[3] - 10000))
+        slice_x, slice_y = area_def.get_area_slices(area_to_cover)
+        self.assertEqual(slice(3, 3709, None), slice_x)
+        self.assertEqual(slice(3, 3709, None), slice_y)
+
+    def test_proj_str(self):
+        from collections import OrderedDict
+        proj_dict = OrderedDict()
+        proj_dict['proj'] = 'stere'
+        proj_dict['a'] = 6378144.0
+        proj_dict['b'] = 6356759.0
+        proj_dict['lat_0'] = 50.00
+        proj_dict['lat_ts'] = 50.00
+        proj_dict['lon_0'] = 8.00
+        area = geometry.AreaDefinition('areaD', 'Europe (3km, HRV, VTC)', 'areaD',
+                                       proj_dict, 10, 10,
+                                       [-1370912.72, -909968.64, 1029087.28,
+                                        1490031.36])
+        self.assertEquals(area.proj_str,
+                          '+a=6378144.0 +b=6356759.0 +lat_0=50.0 +lat_ts=50.0 +lon_0=8.0 +proj=stere')
+        proj_dict['no_rot'] = ''
+        self.assertEquals(area.proj_str,
+                          '+a=6378144.0 +b=6356759.0 +lat_0=50.0 +lat_ts=50.0 +lon_0=8.0 +no_rot +proj=stere')
+
 
 def assert_np_dict_allclose(dict1, dict2):
 
@@ -810,6 +872,7 @@ def assert_np_dict_allclose(dict1, dict2):
 
 
 class TestSwathDefinition(unittest.TestCase):
+
     """Test the SwathDefinition."""
 
     def test_swath(self):
@@ -824,6 +887,17 @@ class TestSwathDefinition(unittest.TestCase):
         self.assertFalse(id(lons1) != id(lons2) or id(lats1) != id(lats2),
                          msg='Caching of swath coordinates failed')
 
+    def test_slice(self):
+        """Test that SwathDefinitions can be sliced."""
+        lons1 = np.fromfunction(lambda y, x: 3 + (10.0 / 100) * x, (5000, 100))
+        lats1 = np.fromfunction(
+            lambda y, x: 75 - (50.0 / 5000) * y, (5000, 100))
+
+        swath_def = geometry.SwathDefinition(lons1, lats1)
+        new_swath_def = swath_def[1000:4000, 20:40]
+        self.assertTupleEqual(new_swath_def.lons.shape, (3000, 20))
+        self.assertTupleEqual(new_swath_def.lats.shape, (3000, 20))
+
     def test_concat_1d(self):
         lons1 = np.array([1, 2, 3])
         lats1 = np.array([1, 2, 3])
@@ -961,9 +1035,9 @@ class TestSwathDefinition(unittest.TestCase):
                          [81.26400756835938, 29.672000885009766, 10.260000228881836]]).T
 
         area = geometry.SwathDefinition(lons, lats)
-        proj_dict = {'no_rot': True, 'lonc': -11.391744043133668,
-                     'ellps': 'WGS84', 'proj': 'omerc',
-                     'alpha': 9.185764390923012, 'lat_0': -0.2821013754097188}
+        proj_dict = {'lonc': -11.391744043133668, 'ellps': 'WGS84',
+                     'proj': 'omerc', 'alpha': 9.185764390923012,
+                     'gamma': 0, 'lat_0': -0.2821013754097188}
         assert_np_dict_allclose(area._compute_omerc_parameters('WGS84'),
                                 proj_dict)
 
@@ -1019,11 +1093,9 @@ class TestSwathDefinition(unittest.TestCase):
 
         res = area.compute_optimal_bb_area({'proj': 'omerc', 'ellps': 'WGS84'})
 
-        np.testing.assert_allclose(res.area_extent, [2253027.149539,
-                                                     -2348379.728104,
-                                                     11687636.846985,
-                                                     2432121.058435])
-        proj_dict = {'no_rot': True, 'lonc': -11.391744043133668,
+        np.testing.assert_allclose(res.area_extent, [-2348379.728104, 2284625.526467,
+                                                     2432121.058435, 11719235.223912])
+        proj_dict = {'gamma': 0.0, 'lonc': -11.391744043133668,
                      'ellps': 'WGS84', 'proj': 'omerc',
                      'alpha': 9.185764390923012, 'lat_0': -0.2821013754097188}
         assert_np_dict_allclose(res.proj_dict, proj_dict)
@@ -1031,6 +1103,7 @@ class TestSwathDefinition(unittest.TestCase):
 
 
 class TestStackedAreaDefinition(unittest.TestCase):
+
     """Test the StackedAreaDefition."""
 
     def test_append(self):
@@ -1189,6 +1262,7 @@ class TestStackedAreaDefinition(unittest.TestCase):
 
 
 class TestDynamicAreaDefinition(unittest.TestCase):
+
     """Test the DynamicAreaDefinition class."""
 
     def test_freeze(self):
@@ -1224,12 +1298,11 @@ class TestDynamicAreaDefinition(unittest.TestCase):
         sdef = geometry.SwathDefinition(lons, lats)
         result = area.freeze(sdef,
                              resolution=1000)
-        np.testing.assert_allclose(result.area_extent, [5016583.682258,
-                                                        -336277.698941,
-                                                        8184964.697984,
-                                                        192456.651909])
-        self.assertEqual(result.x_size, 3)
-        self.assertEqual(result.y_size, 4)
+        np.testing.assert_allclose(result.area_extent,
+                                   [-336277.698941, 5047207.008079,
+                                    192456.651909, 8215588.023806])
+        self.assertEqual(result.x_size, 4)
+        self.assertEqual(result.y_size, 3)
 
     def test_compute_domain(self):
         """Test computing size and area extent."""
@@ -1250,6 +1323,7 @@ class TestDynamicAreaDefinition(unittest.TestCase):
 
 
 class TestCrop(unittest.TestCase):
+
     """Test the area helpers."""
 
     def test_get_geostationary_bbox(self):
